import * as cheerio from "cheerio";
import { ApplicationStatus } from "@/types/application";

interface FetchApplicationsParams {
  page: number;
  filters: {
    status: string;
    programType: string;
    sortBy: string;
  };
  searchQuery: string;
}

const ITEMS_PER_PAGE = 20;

export async function fetchApplications({
  page,
  filters,
  searchQuery,
}: FetchApplicationsParams) {
  // Simulate API delay
  await new Promise((resolve) => setTimeout(resolve, 1000));

  // Generate mock data
  const applications = Array.from({ length: ITEMS_PER_PAGE }, (_, i) => ({
    id: `${page}-${i}`,
    schoolName: `University ${page * ITEMS_PER_PAGE + i}`,
    programName: "Computer Science",
    status: ["submitted", "in-progress", "pending", "accepted", "rejected"][
      Math.floor(Math.random() * 5)
    ] as ApplicationStatus,
    deadline: new Date(Date.now() + Math.random() * 10000000000).toISOString(),
    schoolLogo: "https://images.unsplash.com/photo-1562774053-701939374585?w=800&q=80",
    portalUrl: "#",
  }));

  // Apply filters
  let filteredApplications = applications;
  
  if (filters.status !== "all") {
    filteredApplications = filteredApplications.filter(
      (app) => app.status === filters.status
    );
  }

  if (searchQuery) {
    const query = searchQuery.toLowerCase();
    filteredApplications = filteredApplications.filter(
      (app) =>
        app.schoolName.toLowerCase().includes(query) ||
        app.programName.toLowerCase().includes(query)
    );
  }

  // Sort applications
  switch (filters.sortBy) {
    case "deadline":
      filteredApplications.sort((a, b) => new Date(a.deadline).getTime() - new Date(b.deadline).getTime());
      break;
    case "schoolName":
      filteredApplications.sort((a, b) => a.schoolName.localeCompare(b.schoolName));
      break;
    case "status":
      filteredApplications.sort((a, b) => a.status.localeCompare(b.status));
      break;
  }

  return {
    applications: filteredApplications,
    hasMore: page < 5, // Simulate pagination limit
  };
}

export async function scrapeApplicationUrls(schoolName: string): Promise<string | null> {
  try {
    // Implement rate limiting
    await new Promise((resolve) => setTimeout(resolve, 1000));

    // Construct search URL
    const searchUrl = `https://www.google.com/search?q=${encodeURIComponent(
      `${schoolName} application portal`
    )}`;

    // Fetch and parse HTML
    const response = await fetch(searchUrl);
    const html = await response.text();
    const $ = cheerio.load(html);

    // Extract first result URL
    const firstResult = $("a[href^='/url']").first();
    const url = firstResult.attr("href")?.replace("/url?q=", "").split("&")[0];

    return url || null;
  } catch (error) {
    console.error(`Failed to scrape URL for ${schoolName}:`, error);
    return null;
  }
}